
#include <stdio.h>
#include <stdlib.h>

#include "fossil.h"

const rechts = 0;
const links = 1;

// Permanenten Status speichern
unsigned char motor_status;     // Motoren
unsigned char io_status;        // digitale Eingnge
int mx_status;                  // analoger MX Eingang
int my_status;                  // analoger MY Eingang
long clockticks;
char *sensorbuf;                // Pointer auf Speicherbereich fr Sensordaten


// Diese Funktion schaltet in der globalen variable den Motor
// nr (1-4) ein, mit der Richtung rechts (0) oder links (1).
// Der Motor wird tatschlich erst beim nchsten Durchlauf
// der main-Schleife eingeschaltet !!!
void motor_start (unsigned char nr, char richtung)
{
        motor_status &= !(1 << ((nr * 2) - 2 + richtung));
        motor_status |= (1 << ((nr * 2) - richtung - 1));
        return;
}

// Diese Funktion schaltet in der globalen Variablen den Motor
// nr (1-4) wieder aus.
// Der Motor wird tatschlich erst beim nchsten Durchlauf
// der main-Schleife eingeschaltet !!!
void motor_stop (unsigned char nr)
{
        motor_status &= !(1 << ((nr * 2) - 2));
        motor_status &= !(1 << ((nr * 2) - 1));
        return;
}



long  get_clockticks ()
{
        long foo;
        int foo1, foo2;

        asm {
                xor     ah, ah
                int     0x1a
                mov     cx, 1
                mov     foo1, cx
                mov     foo2, dx
        }

        foo = ((long) foo1) << 16 + foo2;

        return (foo);
}


void store_data ()
{

        return;
}


// Steuerungsfunktion. Hier werden die Steuerbefehle fr das Interface
// vorbereitet.
void steering ()
{

        /*      Ablauf
        =======================

        wiederholen ...
(1)             vorwaerts, bis beide radlaufzaehler >= max_x

(2)             rechtskurve 180 grad, y-bahnzaehler um anzahl der sensoren
                erhhen

(3)             vorwaerts, bis beide radlaufzaehler <= 0

(4)             linkskurve 180 grad

(5)     ... bis y-bahnzaehler >= max_y

 |
 --------- status

        */

        static char status=1;   // Ablaufposition im Programm
        static int  pos_y=0,    // Bahnzaehler
                    pos_x1=0,    // Schrittzaehler Motor1 pro Bahn
                    pos_x2=0;    // Schrittzaehler Motor2 pro Bahn

        const max_x             = 100;
        const max_y             = 4;
        const curve_180_max     = 45;
        static int foo;


        //  I    Schrittsensoren auswerten


        //  II   Analoge Sensorendaten speichern
        store_data ();

        // III   Bewegungen koordinieren
        switch (status)
        {
                case 1:  // Bahn richtung max_x laufen

                        if ((pos_x1 < max_x) && (pos_x2 < max_x))
                        {
                                motor_start (1, rechts);
                                motor_start (2, rechts);
                        } else
                        {
                                motor_stop (1);
                                motor_stop (2);

                                // Vorbereitung auf Status 2
                                status++;
                                foo = pos_x1;
                                pos_x1 = 0;
                        }

                        break;
                case 2: // 180 grad rechtskurve

                        if (pos_x1 < curve_180_max)
                        {
                                motor_start (1, rechts);
                        } else
                        {
                                motor_stop (1);
                                pos_x1 = foo;

                                // Vorbereiten auf Status 3

                        };

                        break;
                case 3:

                     break;
                case 4:

                     break;
                case 5:

                     break;
        }

};



int main ()
{
        unsigned char foo;
        int i;
        long newclockticks;
        int rot=0;
        int bar=0;

        printf ("Initialisiere FOSSIL Treiber\n");
        fossil_init ();
        clockticks = get_clockticks ();

        motor_start (1, rechts);
        for (i=0;i<80;i++) {
/*
                printf ("%dxxx", newclockticks);

                newclockticks = get_clockticks ();
                printf ("%d ticks seit Programmstart vergangen", newclockticks-clockticks);
//                clockticks = newclockticks;
*/
                // Motorstatus neu senden und MX-Status abfragen
                fossil_write (197);
                fossil_write (motor_status);

                io_status = fossil_read ();
                mx_status = (((int) fossil_read ()) << 8) + fossil_read ();

                if (io_status)
                {
                        printf ("rot     ");
                        if (rot)
                                 bar++;
                        rot=0;
                }
                else
                {
                        printf ("schwarz ");
                        rot=1;
                }

                // Motorstatus neu senden und MX-Status abfragen
/*                fossil_write (201);
                fossil_write (motor_status);

                io_status = fossil_read ();
                my_status = (((int) fossil_read ()) << 8) + fossil_read ();
*/

                // Steuerungsfunktion aufrufen
        //        steering ();
        };

        motor_status=0;
        fossil_write (201);
        fossil_write (motor_status);
        printf ("Deinitialisiere FOSSIL-Treiber...\n");
        fossil_done ();

        return;
}

